#include "armadillo/armadillo.h"

#include "root/gdefs.h"
//#include config_fred.h"

#include "geom/aritm.h"


//---------------------------------------------------------
/*---ecrit le contenu du vecteur dans un fichier --
format : taille, contenu

*/


void Ecrit_fichier(cx_vec & V, ofstream & fich)
{

  fich<<V.n_rows<<endl;

 for(int i=0;i<V.n_rows;i++)
   fich<<V(i)<<endl;

}
//-----------------------------------------------
void Ecrit_fichier(cx_vec &V,const char * nom_fich)
{
  cout<<" Ecrit cx_vec dans fichier: "<<nom_fich<<endl;
  ofstream fich(nom_fich);
  Ecrit_fichier(V,fich);

}


//------------------------------------------
void Lit_fichier(cx_vec &V,ifstream & fich)
{

  int size;
  fich>>size;

  V.set_size(size);

  for(int i=0;i<size;i++)
    fich>>V(i);

}
//--------------------------------------------
void Lit_fichier(cx_vec &V,const char * nom_fich)
{
  ifstream fich(nom_fich);
  Lit_fichier(V,fich); 
}


//============================
/*
Entrée:
 matrice complexe ou reelle M
Sortie:
 val_p, vect _p  ordonnées par module croissant
 */

void Diago(cx_vec & val_p,cx_mat vect_p, cx_mat & M)
{
  eig_gen(val_p, vect_p, M); // diagonalise
 

  // cout<<"A3="<<A3<<endl<<" eigenvalues="<<val_p<<endl;
  // for(int i=0;i<10;i++)
  //   cout<<norm(val_p(i))<<", "<<flush;
  // cout<<endl;


  //... ordonne les valeurs propres (et vect p)  par module decroissant

  uvec indices = sort_index(abs(val_p)); // liste des indices
  val_p=val_p(indices); // vecteur ordonné
  vect_p=vect_p.cols(indices); // vecteur ordonné  
}


void Diago(cx_vec & val_p,cx_mat vect_p, mat & M)
{
  eig_gen(val_p, vect_p, M); // diagonalise
 

  // cout<<"A3="<<A3<<endl<<" eigenvalues="<<val_p<<endl;
  // for(int i=0;i<10;i++)
  //   cout<<norm(val_p(i))<<", "<<flush;
  // cout<<endl;


  //... ordonne les valeurs propres (et vect p)  par module decroissant

  uvec indices = sort_index(abs(val_p)); // liste des indices
  val_p=val_p(indices); // vecteur ordonné
  vect_p=vect_p.cols(indices); // vecteur ordonné  
}

//=======================================
int Recherche_indice_du_max(vec &V)
{
 uvec indices = sort_index(V); // liste des indices
 return indices(V.size());
}


//==========================

/*===================================
 Permutte circulaire de (d) elements
  */
vec Permut_circ2(vec &V,int d)
{

    vec temp(V.size());

    for (int i=0;i<V.size();i++)
        temp(i)=V(ireste((i-d),V.size()));
    return temp;
}
//=========================== GRAPHISME ============================

#include <TCanvas.h>
#include <TGraph.h>
#include <TPolyMarker.h>
#include <TEllipse.h>
#include <TH1.h>
#include <TROOT.h>

#include <TStyle.h>
#include <TView.h>

#include <TPolyMarker3D.h>
 
//========= dessin ========================
TH1D*  Dessin(vec &V,const char * titre,int superp,const char *opt,int opt_log)
{
    return Dessin(V,"","",1,V.size(),titre,superp,opt,-1111,-1111,1,opt_log);
}

//========= dessin ========================
TH1D*  Dessin(vec &V,const char * titre,const char *opt)
{
    return Dessin(V,"","",1,V.size(),titre,0,opt);
}

//========= dessin ========================
TH1D*  Dessin(Col<int> &V,const char * titre,const char *opt)
{
	vec V2=conv_to<vec>::from(V);
    return Dessin(V2,"","",1,V.size(),titre,0,opt);
}
//========= dessin ========================
TH1D*  Dessin(vec &V,const char * xtitre,const char * ytitre,double xmin,double xmax,const char * titre, int superp,const char *opt,double ymin, double ymax,int col,int opt_log)
{



    //... recherche si il y a deja une fenetre "titre"...
    TCanvas *c=(TCanvas*)gROOT->GetListOfCanvases()->FindObject(titre);
    if (c==0) // il n'y en a pas déjà, crée nouvelle fenetre
    {
//		cout<<"cree nouvelle fenetre dans vecteur.cc, titre="<<titre<<endl;
        c = new TCanvas(titre,titre,500,500);
        c->SetFillColor(0);
        superp=0;
    }
    else
    {
//		cout<<"fenetre deja existante:"<<titre<<endl;
    }



    return Dessin(V,xtitre, ytitre,xmin, xmax, titre, c,superp,opt,ymin,ymax,col,opt_log);
}
//========= dessin ========================
TH1D *  Dessin(vec & V,TCanvas * c)
{
    return Dessin(V,"","",1,V.size(),"Vecteur", c,0);
}
//========= dessin ========================
//  c : fenetre ou dessiner
// superp: =0 si premier figure, 1,2.. si superposition (style de ligne)
// ymin, ymax pour fixer. (-1111 si libre)
TH1D*  Dessin(vec& V,const char * xtitre,const char * ytitre,double xmin,double xmax,const char * titre,TCanvas * c,int superp,const char* opt,double ymin, double ymax,int col,int opt_log)
{
    int nx(V.size());



	ostringstream titre_histo; titre_histo<<titre<<superp<<ends;

    TH1D *h1=(TH1D*) gROOT->FindObject(titre_histo.str().c_str());

    if(h1) // "titre" existe dejà
	{
//		cout<<"on detruit h1 dans vecteur.cc"<<endl;
		delete h1;
	}


    h1=new TH1D(titre_histo.str().c_str(),titre,nx,xmin,xmax);



    h1->SetStats(kFALSE); // pas de panneau stat
    h1->SetMinimum(ymin);
    h1->SetMaximum(ymax);
    h1->SetXTitle(xtitre);
    h1->SetYTitle(ytitre);


    for (int  i=0;i<nx;i++) // remplissage
    {
        double z=V[i];
        h1->SetBinContent(i+1,z);

    }


//---- definit le marker P ----
    h1->SetMarkerColor(1);
    h1->SetMarkerStyle(6); // 3:etoile  6: petit cercle
    h1->SetLineStyle(1); // 1=solid, 2=dash, 3=dash-dot, 4=dot-dot
    h1->SetLineColor(col);
	h1->SetLineWidth(3); //1,2,3

    c->cd();



    if (superp==0)
        h1->Draw(opt); // L: ligne  P: marker
    else
    {
        int style=1;

        if (superp>=1)
            style=(superp-1%4)+1;
        h1->SetLineStyle(1);//style); // 1=solid, 2=dash, 3=dash-dot, 4=dot-dot
		col=style;
        h1->SetLineColor(col);
        h1->Draw("same");
    }

    if (opt_log) c->SetLogy(1); //echelle LOG


    c->Update();
    return h1;
}
//========= dessin 2D (x[i],y[i] = objet)========================
TCanvas *  Dessin(vec &x,vec &y,const char * xtitre,const char * ytitre,const char * titre)
{

    if (x.size() !=y.size())
    {
        cerr<<"Il faut x.I=y.I dans Vecteur.cc"<<endl;
        return 0;
    }

    TCanvas *c = new TCanvas(titre,titre,400,400);
    c->SetFillColor(0);
    int nx(x.size());


    float *xx=new float[nx];
    float *yy=new float[nx];

    for (int i=0;i<nx;i++)
    {
        xx[i] =x[i];
        yy[i] =y[i];
    }


    TGraph  *gr= new  TGraph(nx,xx,yy);

    /*
    (gr->GetXaxis())->SetLabel(xtitre);
    (gr->GetYaxis())->SetLabel(ytitre);
    */
    c->cd();

    gr->Draw("AP"); // A: axis L:ligne , P=marker *: etoile

    c->Update();

    return c;
}

//========= dessin 3D (x[i] = objet, y[i], z[i])========================
TCanvas *  Dessin(vec &x,vec &y,vec &z, const char * xtitre,const char * ytitre,const char * ztitre,const char * titre)
{

    if ((y.size() !=x.size())||(z.size() !=x.size()))
    {
        cerr<<"Il faut taille y.I=I dans Vecteur.cc"<<endl;
        return 0;
    }

    TCanvas *c = new TCanvas(titre,titre,400,400);
    c->SetFillColor(0);
    int nx(x.size());

    
 // creating view
    TView *view = TView::CreateView(1,0,0);

    double xmin=x.min(),ymin=y.min(),zmin=z.min(),xmax=x.max(),ymax=y.max(),zmax=z.max();
   view->SetRange( xmin,ymin,zmin,xmax,ymax,zmax);

   TPolyMarker3D *pm3d = new TPolyMarker3D(x.size());



   for( int i = 0; i <x.size(); i++ )
      {
	  double xp=x[i], yp= y[i], zp= z[i] ;
	  //double xp=0.5*(1.+cos(2.*Pi_*i/(1.*I))),  yp=0.5*(1.+sin(2.*Pi_*i/(1.*I))) , zp=0;
	  pm3d->SetPoint( i,xp,yp,zp);
      }

      // set marker size, color & style
      pm3d->SetMarkerSize( 1 );
      pm3d->SetMarkerColor(2);
      pm3d->SetMarkerStyle( 1 );
      
      //draw
      c->cd();
      pm3d->Draw();

      

   

    c->Update();

    return c;
}
//========= dessin sur cercle========================
void  Dessin_phases(vec &V,const char * titre)
{
    TCanvas *c = new TCanvas(titre,titre,400,400);
    c->SetFillColor(0);
    int nx(V.size());


    float *x=new float[nx+2];
    float *y=new float[nx+2];

    for (int i=0;i<nx;i++)
    {
        x[i] = cos(V[i]);
        y[i] = sin(V[i]);
    }
    x[nx]=1;
    y[nx]=1; // pour cadrer
    x[nx+1]=-1;
    y[nx+1]=-1;

    TGraph  *gr= new  TGraph(nx+2,x,y);

    gr->Draw("A*");
    TEllipse *e=new  TEllipse(0,0,1);
    e->Draw();

    c->Update();

}


//========= dessin  sur colonne a l abcisse x========================
void  Dessin_col(vec &V,double xx, TCanvas *c,int col, int style)
{
    c->cd();
    float *x=new float[V.size()];
    float *y=new float[V.size()];

    double xm,ym,xM,yM;
    c->GetRange(xm,ym,xM,yM); //limites de la fenetre

    for (int i=0;i<V.size();i++)
    {
        x[i] = xx;
        y[i] = V[i];

        if (y[i]>yM) y[i]=yM; //si dépassement vertical
        if (y[i]<ym) y[i]=ym;
    }



    TPolyMarker *pp= new TPolyMarker(V.size(),x,y);

    pp->SetMarkerColor(col);
    pp->SetMarkerStyle(style ); //6: point  8: petit cercle
    pp->Draw();


    /*
    TGraph  *gr= new  TGraph(I,x,y);
    gr->Draw("sameAC*");
    */
    //   c->Update();
}



//========= dessin ========================
// dessin des points complexes dans le plan complexe
TCanvas *  Dessin(cx_vec &V,const char * titre)
{
 //... recherche si il y a deja une fenetre "titre"...
  TCanvas *c=(TCanvas*)gROOT->GetListOfCanvases()->FindObject(titre);
 if(c==0) // il n'y en a pas déjà, crée nouvelle fenetre 
    {
      c = new TCanvas(titre,titre,400,400);
      c->SetFillColor(0);
    
    }
 else
   {
//      cout<<"fenetre deja existante:"<<titre<<endl;
      c->Clear();
   }


  
  c->cd();
  // double xmin=(*this).Real().Minimum(),  xmax=(*this).Real().Maximum();
  // double ymin=(*this).Imag().Minimum(),  ymax=(*this).Imag().Maximum();
  double xmin(-1),xmax(1),ymin(-1),ymax(1);
  
  c->DrawFrame(xmin,ymin,xmax,ymax,titre);
  
  for (int i=0;i<V.size();i++) 
   {
     double x = (double)real(V[i]);
     double y= (double)imag(V[i]);
	 
     point(x,y,kRed,8);// color,style


      
   }
  
  
  
  //-------- dessin du cercle unité -----------------
  circle(0,0,1);

  //-----------------------------------
 c->Update();

 return c;
}





/*
//==================================
// Dessine les histogrammes des phases et des modules des elements du vecteur
void Dessin_histo_phases_modules(cx_vec &V,double module_min, double module_max)
{
    cout<<"Nbre de valeurs ="<<V.size()<<endl;
    
    double phase_min=-Pi_, phase_max=Pi_; 
    int nbin=V.size()/3+1;
    vec h_phases=((Phase(V)).Remplit_histo(phase_min,phase_max,nbin,"histo of phases"); // histogramme
    h_phases.Dessin("#phi","P",phase_min,phase_max,"phases");
    
    
   
   // int nbin=(*this).I/20+1;
    Vecteur mod=(V.Module());
   module_max=mod.Maximum();
   module_min=mod.Minimum();
    Vecteur h_modules=mod.Remplit_histo(module_min,module_max,nbin,"histo of modules"); // histogramme
    h_modules.Dessin("|z|","P",module_min,module_max,"modules");
    
      
      
}

*/





//========= Matrices ===================






//========= dessin ========================
TCanvas*  Dessin(mat &M,const char * titre,const  char *option)
{

	return Dessin(M,"i",1,M.n_cols,"j",1,M.n_rows,titre,option);
}
//========= dessin ========================
TCanvas*  Dessin(mat &M,const char * xtitre,double xmin,double xmax,const char * ytitre,double ymin,double ymax,const char * titre,const char * option,double zmin, double zmax)
{


 //... recherche si il y a deja une fenetre "titre"...
  int superp=0;

  TCanvas *c;
  c=(TCanvas*)gROOT->GetListOfCanvases()->FindObject(titre);

 if(c==0) // il n'y en a pas dÃ©jÃ , crÃ©e nouvelle fenetre
   { cout<<"cree fenetre dans matrice.cc"<<endl;
	   c = new TCanvas(titre,titre,500,500);
      c->SetFillColor(0);

      superp=0;
    }
 else
   {
      cout<<"fenetre deja existante:"<<titre<<endl;
   }



 Dessin(M,xtitre,xmin,xmax,ytitre,ymin,ymax,titre,option,c,superp,zmin,zmax);

return c;
}
//========= dessin ========================
//  c : fenetre ou dessiner
// superp: =0 si premiere figure, 1 si superposition
// zmin, zmax pour fixer. (-1111 si libre)
TCanvas*  Dessin(mat &M,const char * xtitre,double xmin,double xmax,const char * ytitre,double ymin,double ymax,const char * titre,const  char * option,TCanvas * c,int superp,double zmin, double zmax)
{ //-------


char options_2[30];
strcpy(options_2,option);
if(superp!=0)
  {
   strcat(options_2,"same");
   c->Clear();
  }

//-----------------------

 int nx(M.n_cols),ny(M.n_rows);



 TH2D *h=new TH2D(titre,titre,nx,xmin,xmax,ny,ymin,ymax);

 h->SetXTitle(xtitre);
 h->SetYTitle(ytitre);

 for( int i=0;i<M.n_rows;i++)  // remplissage
   for(int j=0;j<M.n_cols;j++)
     {
		 double z=M(i,j);


		 h->SetCellContent(i+1,j+1,z);
     }
  //-----palette de gris -------pour:TH2F -SetContour(7); Draw("cont" ou "surf2");------
 /*
   Int_t colors[]={1,0,19,18,17,15,13,1,1}; // (ordre:des indices dans la palette 4,5,6,7,0,1,3)
   gStyle->SetPalette(8,colors); // classe TStyle
h->SetContour(7);
 */

//gStyle->SetPalette(9,0); h->SetContour(10); // classe TStyle, palette de gris.
 //gStyle->SetPalette(30,0); h->SetContour(30); // classe TStyle, palette de gris.


    gStyle->SetPalette(1,0); // jolie palette bleu->rouge
    gStyle->SetPadGridX(kTRUE); // grilles en X,Y
    gStyle->SetPadGridY(kTRUE);
    gPad->SetCrosshair(); // dessine une croix sous la souris



// h->SetEntries(1); // pour voir les lignes avec "cont".

//zmin=1e-4; cout<<"Min 1e-4 dans matrice.cc"<<endl;

// zmax=1; //@@
 h->SetMaximum(zmax);
  h->SetMinimum(zmin);

 c->Clear(); // efface les dessins sous-jacents
 h->SetStats(kFALSE); // pas de panneau stat

//..... en 3D: car cont ne marche pas!
 /*
gPad->SetPhi(0.0);    // angle de position (action sur le pad actuel)
gPad->SetTheta(90.0);
h->Draw("asurf2");  // a: sans graduations
 */

 //c->SetLogz(1); // echelle log

 
//....... peu de divisions sur les axes ....
// h->GetXaxis()->SetNdivisions(5);  h->GetYaxis()->SetNdivisions(5);  h->GetZaxis()->SetNdivisions(5);

 c->cd();
 h->Draw(options_2);


 //-------- dessin du cercle unité -----------------
 // circle(0,0,1);


 c->Update();
return c;
}




//--------------------------
/*
Dessine les lignes de niveau de || (z-P)^-1 ||_2  dans le plan complexe z.

Algo: ameliorer l'algo en  remplacant P par une matrice Triangulaire: P= O T O^-1
 avec O orthogonale,
ref: Threfethen's book.

 
 */
void Dessin_Pseudo_Spectre(mat & M,double xmin,double xmax,double ymin,double ymax)
{ 


	int Nx(200), Ny(200); // >1, taille du tableau de z=x+iy 

	mat R(Nx,Ny);// tableau de la norme de la resolvente

	int N=M.n_rows;
	if(M.n_cols!=N)
	{
		cout<<"Pb taille matrice dans armadillo.cc"<<endl;
		exit(1);
	}

	for(int ix=0;ix<Nx;ix++)
		for(int iy=0;iy<Ny;iy++)
		{
			double x=ix/(Nx-1.)*(xmax-xmin)+xmin;
			double y=iy/(Ny-1.)*(ymax-ymin)+ymin;
			Complex z=x+I_*y;
			
			mat Id=mat(N,N,fill::eye); // matrice identité
			vec S=svd(z*Id-M); // valeurs singulieres , ordre decroissant
			double smin=S(N-1); // la plus petite

			if(smin>0)
				R(ix,iy)=1./smin; //norme de la resolvente en z
			else
				// sinon c'est une val propre
				R(ix,iy)=0;
		} // for ix,iy


//....

	Dessin(R,"Re(z)",xmin,xmax,"Im(z)",ymin,ymax,"pseudo-spectre");	




}	





//=====================================

// OPERATIONS SUR LES VECTOR



 

//=====================
// A typical alternative that forces a reallocation is to use swap:	
/*
void Clear(vector & V)
{
vector<T>()->swap(V);
}
*/






//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<pair<int,int>> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<" , "<<flush;
      sortie<<"("<<V[i].first<<"/"<<V[i].second<<")="<<V[i].first*1./V[i].second<<" ";
    }
  cout<<"]"<<endl;

    return sortie;
}

//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<pair<double,double>> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<" , "<<flush;
      sortie<<"("<<V[i].first<<";"<<V[i].second<<")";
    }
  cout<<"]"<<endl;

    return sortie;
}


//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<double> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<","<<flush;
      sortie<<V[i]<<flush;
    }
  cout<<"]"<<endl;

    return sortie;
}

//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<Complex> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<","<<flush;
      sortie<<V[i]<<flush;
    }
  cout<<"]"<<endl;

    return sortie;
}


//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<int> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<","<<flush;
      sortie<<V[i]<<flush;
    }
  cout<<"]"<<endl;

    return sortie;
}

//===================
/*
ostream & operator <<(ostream & sortie,  const vector<int> & n)
{
	for(int i=0; i<n.size(); i++)
		sortie<<","<<n[i];
    return sortie;
}
*/
//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<unsigned char> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<","<<flush;
      sortie<<V[i]<<flush;
    }
  cout<<"]"<<endl;

    return sortie;
}
//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<vector<int> > & L)
{
  for(int l=0;l<L.size();l++)
    {
      sortie<<L[l];
    }

    return sortie;
}

//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<string> & V)
{

 sortie<<"["<<flush;

  for(int i=0;i<V.size();i++)
    {
      if(i>0)
	sortie<<","<<flush;
      sortie<<V[i]<<flush;
    }
  cout<<"]"<<endl;

    return sortie;
}

//==========================
// Affiche vector
ostream & operator <<(ostream & sortie,  const vector<vector<string> > & L)
{
  for(int l=0;l<L.size();l++)
    {
      sortie<<L[l];
    }

    return sortie;
}

//=============================
bool Subset(vector<int> V1,vector<int> V2)
{
/*
sortie:
   1 si V1 est un sous ensemble de V2
   0 sinon.

Exemple:
   vector<int> V1; V1.push_back(2); V1.push_back(1);  V1.push_back(3);
   vector<int> V2; V2.push_back(1); V2.push_back(1);  V2.push_back(3);
   vector<int> V3; V3.push_back(3); V3.push_back(1); 

   cout<<V1<<endl;    cout<<V2<<endl;    cout<<V3<<endl;

   cout<<"V1 in V2 :"<<Subset(V1,V2)<<endl;
   cout<<"V3 in V1 :"<<Subset(V3,V1)<<endl;
   cout<<"V3 in V2 :"<<Subset(V3,V2)<<endl;


 */

  sort(V1.begin(),V1.end());
  sort(V2.begin(),V2.end());
  //  cout<<"V1 sorted:"<<V1<<endl;
  //cout<<"V2 sorted:"<<V2<<endl; 

  bool res= includes(V2.begin(),V2.end(), V1.begin(), V1.end());
//cout<<"res="<<res<<endl;
 return  res;
}

//===========================
vector<int> Sym_diff(vector<int> A,vector<int> B)
{
/*
Renvoit elements presents dans un seul vecteur (avec multiplicité)

Exemple

vector<int> V1; V1.push_back(2); V1.push_back(1);  V1.push_back(3);
vector<int> V2; V2.push_back(1); V2.push_back(1);  V2.push_back(4);
cout<<V1<<endl;
cout<<V2<<endl; 
cout<<Sym_diff(V1,V2)<<endl;

renvoit
[2,1,3]
[1,1,4]
[1,2,3,4]
 */

  sort(A.begin(), A.end());
  sort(B.begin(), B.end());
  // allocate the smallest size of A,B as maximum size
  vector<int> c(A.size() < B.size() ? B.size() : A.size());
  vector<int>::iterator i;
  i = set_symmetric_difference(A.begin(), A.end(), B.begin(), B.end(), c.begin());
  return vector<int>(c.begin(), i);
}


//===========================
vector<int> diff(vector<int> A,vector<int> B)
{
/*
Renvoit elements presents A et pas dans B

Exemple

vector<int> V1; V1.push_back(2); V1.push_back(1);  V1.push_back(1);  V1.push_back(3);
vector<int> V2; V2.push_back(1);   V2.push_back(4);
cout<<V1<<endl;
cout<<V2<<endl; 
cout<<diff(V1,V2)<<endl;

renvoit
[2,1,1,3]
[1,4]
[1,2,3]
 */

  sort(A.begin(), A.end());
  sort(B.begin(), B.end());
  // allocate the smallest size of A,B as maximum size
  vector<int> c(A.size() < B.size() ? B.size() : A.size());
  vector<int>::iterator i;
  i = set_difference(A.begin(), A.end(), B.begin(), B.end(), c.begin());
  return vector<int>(c.begin(), i);
}

//===========================
vector<int> Intersection(vector<int> A,vector<int> B)
{
/*
Renvoit elements presents dans les deux vecteurs (avec multiplicité)

Exemple

vector<int> V1; V1.push_back(2); V1.push_back(1);  V1.push_back(3);
vector<int> V2; V2.push_back(1); V2.push_back(1);  V2.push_back(4);
cout<<V1<<endl;
cout<<V2<<endl; 
cout<<Intersection(V1,V2)<<endl;

renvoit
[2,1,3]
[1,1,4]
[1]
 */

  sort(A.begin(), A.end());
  sort(B.begin(), B.end());
  // allocate the smallest size of A,B as maximum size
  vector<int> c(A.size() < B.size() ? B.size() : A.size());
  vector<int>::iterator i;
  i = set_intersection(A.begin(), A.end(), B.begin(), B.end(), c.begin());
  return vector<int>(c.begin(), i);
}

//===========================
vector<int> Union(vector<int> A,vector<int> B)
{
/*
Renvoit union des elements des  deux vecteurs

Exemple

vector<int> V1; V1.push_back(2); V1.push_back(1);  V1.push_back(3);
vector<int> V2; V2.push_back(1); V2.push_back(1);  V2.push_back(4);
cout<<V1<<endl;
cout<<V2<<endl; 
cout<<Union(V1,V2)<<endl;

renvoit
[2,1,3]
[1,1,4]
[1,1,2,3,4]
 */

  sort(A.begin(), A.end());
  sort(B.begin(), B.end());
  // allocate the smallest size of A,B as maximum size
  vector<int> c(A.size() < B.size() ? B.size() : A.size());
  vector<int>::iterator i;
  i = set_union(A.begin(), A.end(), B.begin(), B.end(), c.begin());
  return vector<int>(c.begin(), i);

}

//===========================
// renvoit true si j est pas contenu dans V, false si contenu
int Not_find_j_in_V(int j,vector<int> V)
{
  return (find(V.begin(), V.end(), j) == V.end());
}



//====================
//Conversions
vec vector_to_vec(vector<double> &v)
{
	vec V(v.size());
	for(int i=0;i<v.size();i++)
		V[i]=v[i];
	return V;

}

//====================
//Conversions
cx_vec vector_to_vec(vector<Complex> &v)
{
	cx_vec V(v.size());
	for(int i=0;i<v.size();i++)
		V[i]=v[i];
	return V;

}
