// -*- mode:C++ ; compile-command: ". ~/.mon_bashrc; compp ~/c++/syst_dynamiques/logistic_map logistic_map.cc" -*-
/*
Citation:
Titre: 
Auteur: Fred
Date: aout 2013.
Abstract:
Application logitique
f(x) = mu * x * (1-x)
Classement: 
*/



#include "config_fred.h"
#include <iostream>
using namespace std;

#include <string.h>
#include <sstream>
#include <iomanip>


#include <TROOT.h>
#include <TSystem.h>

#include <TF1.h>
#include <TApplication.h>
#include <TRandom.h>
#include <gsl/gsl_errno.h>


#include "root/gdefs.h"
#include "armadillo/armadillo.h"

//=======================
TCanvas * c_xy; // fenetre du graphe 
TCanvas * c_mu_x; // fenetre du diag de phase
TCanvas * c_z; // plan z complexe
TCanvas *c_mu; // plan mu complexe



//======================
double f(double x,double mu)
{
  return mu*x*(1-x);
}
//============================
Complex f(Complex x,Complex mu)
{
  return mu*x*(1.-x);
}
//======================
// application inverse
// entrée: s=0,1
// n: nbre d'iterations
Complex f_inv(Complex x0,Complex mu,int s,int n)
{
  if(n<=0)
    return 0;

  Complex x1=0.5*(1.+(2.*s-1.)*sqrt(1.-4.*x0/mu));

  //.... dessin points sur axe x ....
  c_z->cd();
  int col=BLACK;
  if(n<=10)
    col=GREEN;
  point(real(x1),imag(x1),col);

  //...................
  for(int s1=0;s1<=1;s1++)
    {
      f_inv(x1,mu,s1,n-1);
    }
  return x1;
}

//================= fonction f pour root ============================
double fonction_x(double * xx,double *par)
{
   double x=xx[0];

   int n=par[1]; //: temps
   double mu=par[0];

   for(int it=1;it<=n; it++)
     x=f(x,mu);
   return x;
}

//======================
// entrée mu, 
//         n: temps (iteration de f)
void Dessin_graphe(double mu,int n)
{
  c_xy->Clear();
  //  c_xy->DrawFrame(0,0,1,1,"f");

  //----------------------------
  int npar=2;// nbre de parametres
  TF1 *fr=new TF1("Logistic",fonction_x,0,1,npar); 
  fr->SetParameters(mu,n);
  fr->SetMaximum(1);
  fr->Draw();

  //------- dessin y=x -----
  TF1 *f2=new TF1("","x"); 
  f2->Draw("same");

  //--------------------
  // c_xy->Update();

}
//=====================================
void Dessin_trajectoire(double mu, int n, double x0,int opt_animation=0)
{
  Dessin_graphe(mu,1); // dessin du graphe

  
  
  //-------------------------------
  for(int it=1;it<=n;it++)
  {
      double x1=f(x0,mu);
      //.... dessin traits ....
      int style=1;
      int col=RED;
      if(it>n/2)	col=BLUE;
      move2(x0,0); draw2(x0,x1,col,1,style);
      draw2(x1,x1); draw2(x1,0);
      //.... dessin points sur axe x ....
      point(x0,0,BLACK,8,1);

      if(opt_animation) // on cree un gif animé
      {
		  TMarker tm=TMarker(x1,0,8);
		  tm.SetMarkerColor(RED);
		  tm.Draw();
	  
		  ostringstream is; is<<"#mu="<<mu<<" n="<<it;
		  Animation(c_xy,it,1,n,"traj",70,is.str().c_str());
	  
      }
      //.....
      x0=x1;
  }

  c_xy->Update();
}

//============================
/*
  trajectoire pour condition initiale donnée et en fonction de mu
*/
void Animation_trajectoire()
{
	double mu1=3, mu2=4;
	int n=10000; // temps
	double x0=0.1; // position initiale

	int N=500;

	for(int it=0;it<=N;it++)
    {
		double mu=mu1+(mu2-mu1)*it*1./N;
		Dessin_trajectoire(mu,n,x0);
		//............
		ostringstream is; is<<"#mu="<<mu;
		Animation(c_xy,it,0,N,"film",70,is.str().c_str());

    }


}

//===============================
// graphe en fonction de mu
void Dessin_ensemble_attractif()
{

	double mu1=0, mu2=4;
	int n1=500, n2=2*n1; // temps

	int N=400; // nbre de points en mu

 


	c_mu_x=new TCanvas("f2","f2",400,400);
	TH1F* th= c_mu_x->DrawFrame(mu1,0,mu2,1,"f");
	th->SetXTitle("#mu");
	th->SetYTitle("x");

	for(int imu=0;imu<=N;imu++)
    {
		double mu=mu1+(mu2-mu1)*imu*1./N;
      
		double x0=0.1; // position initiale

		for(int it=1;it<=n2;it++)
		{
			double x1=f(x0,mu);
			x0=x1;

			if(it>n1) // on dessine
				point(mu,x0,BLUE);
		}// for it
		c_mu_x->Update();
		Animation(c_mu_x,imu,0,N,"film",10);
	
    } // for imu


}


//================================
// graphe en fonction de mu
void Dessin_ensemble_repulsif(Complex mu, int n, Complex x0)
{
	c_z->Clear();
	double xmin=0,xmax=1,ymin=-0.5,ymax=0.5;
	TH1F* th= c_z->DrawFrame(xmin,ymin,xmax,ymax,"f");
	th->SetXTitle("Re(z)");
	th->SetYTitle("Im(z)");
	ostringstream is; is<<"#mu="<<mu;
	th->SetTitle(is.str().c_str());

	//----dessin ens repulsif ---------------------------
	f_inv(x0,mu,0,n);

	//--- dessin ens attracteur -----------
	n=5000;

	for(int it=1;it<=n;it++)
    {
		Complex x1=f(x0,mu);
		if(abs(x1)>1000)
			break;
		if(it>500)
			point(real(x1),imag(x1),BLUE,8);
		//.....
		x0=x1;
    }

	//---------------------------
  
	c_z->Update();
}

//======================================
void Dessin_Mandelbrot()
{
	int N=200; // pas de discretisation du plan
	mat M(N,N);
	double xmin=-3,xmax=5,ymin=-1.5,ymax=1.5;

	c_mu=new TCanvas("Mandel","Mandel",400,400);
	TH1F* th= c_mu->DrawFrame(xmin,ymin,xmax,ymax,"Mandelbrot");
	th->SetXTitle("Re(#mu)");
	th->SetYTitle("Im(#mu)");

	//.............................

	for(int i=0;i<N;i++)
		for(int j=0;j<N;j++)
		{
			double xmu=xmin+(xmax-xmin)*i*1./N;
			double ymu=ymin+(ymax-ymin)*j*1./N;

			Complex mu(xmu,ymu);

			//.... trajectoire de x=1/2 ....
			Complex x0=0.5;

			int n=1000;
			int nt=n;
			for(int it=1;it<=n;it++)
			{
				Complex x1=f(x0,mu);
				if(abs(x1)>1000)
				{
					nt=it;
					break;
				}
				//.....
				x0=x1;
			}

      
			if(nt==n) // on est dans l'ens de Mandelbrot
				point(real(mu),imag(mu),BLACK);
	
      
			M(i,j) = log(nt);

		}

	c_mu->Update();
  
	Dessin(M,"Re(#mu)",xmin,xmax,"Im(#mu)",ymin,ymax,"Mandelbrot");
}


//=================================
//
void Animation_ens_repulsif()
{
    c_z=new TCanvas("f3","f3",400,400);
   
    

  double mu1=1,mu2=5;
  int N=400; //nbre de pas en mu
  int n=15; // temps

  //---- Dessin ensemble Mandelbrot --
 
  Dessin_Mandelbrot();
  
  //----------------------------
  int imin=1;

  for(int it=imin;it<=N;it++)
    {
      Complex mu=mu1+(mu2-mu1)*it*1./N;
      mu=mu+Complex(0,0.2);

      Complex x0=0.1; // position initiale
      Dessin_ensemble_repulsif(mu,n,x0);

      //..Dessin du point sur l'ens de Mandelbrot .........
      c_mu->cd();
      TMarker tm(real(mu),imag(mu),8);
      tm.SetMarkerColor(RED);
      tm.Draw();
      c_mu->Update();
      //............

      Animation(c_z,it,imin,N,"film",10);
      Animation(c_mu,it,imin,N,"mandel",10);

    }
}

//===Main================================================

int main(int argc, char **argv)
{

    gsl_set_error_handler_off();// evite les underflow avec Bessel()

    TApplication theApp("App", &argc, argv);

    gROOT->SetStyle("Plain");

    //    gROOT->SetBatch(kTRUE); // mode Batch cad sans sortie ecran (+ rapide)

    gRandom->SetSeed(0);  // initialise sur l'horloge (#include <TRandom.h>)


    //------ Initialisation --------

    c_xy=new TCanvas("f","f",5);
    
    


    //----dessin du graphe -----------------------------
    /*
    double mu=3;
    int n=1; // temps
    Dessin_graphe(mu,n);
	*/

    //---- dessin/animation d'une trajectoire -----------
    /*
    double mu=4;
    int n=200; // temps
    double x0=0.1; // position initiale
    int opt_animation=0 ;
    Dessin_trajectoire(mu,n,x0,opt_animation);
    */

    //-------- Animation:  trajectoire pour condition initiale donnée et en fonction de mu
    
	Animation_trajectoire();



    //-------- Dessin "diagramme de phase" cad ensemble attractif en fonction de mu ----
    //  Dessin_ensemble_attractif();


    //---------- Dessin de l'ensemble repulsif dans le plan complex x -------
/*    
    c_z=new TCanvas("f3","f3",5);
    Complex mu=4.5;
    int n=17; // temps
    Complex x0=0.1; // position initiale
    Dessin_ensemble_repulsif(mu,n,x0);
*/  

    //--- Animation: Dessin de l'ens repulsif en fonction de mu  

	//   Animation_ens_repulsif();

    //---------------------
    //     Dessin_Mandelbrot();

    //--------------------------
    //    Recree_Animation(1,10,"film",30);
    /// ----Donne la main a l'utilisateur ---
    theApp.Run();

    return 0;
}


